/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

/*
 * Main driver loop for BPF version.
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include <clack/c/clack_types.h>
#include <net/ethernet.h>

/*
 * Imports. The fds are handed off from the FromBPF device so that
 * we can call select on them to determine when packets are available
 * for delivery.
 */
extern unsigned char	ethaddr0[ETHER_ADDR_LEN];
extern unsigned char	ethaddr1[ETHER_ADDR_LEN];
extern int		fd0, fd1;
extern void		callback0(), callback1();
extern void		schedule();

#ifdef MEASURE
/*
 * More Imports!
 */
#include <clack/c/stats.h>
#endif

/*
 * The real Export!
 */
int
main(int argc, char* argv[])
{
	fd_set		mask;
	struct timeval	tv;
	int		n, setsize;

        printf("Clack kernel\n");
        printf("eth0: "); print_ether(ethaddr0); printf("\n");
        printf("eth1: "); print_ether(ethaddr1); printf("\n");

	setsize = ((fd0 > fd1) ? fd0 : fd1);
	setsize++;

        while (1) {
		FD_ZERO(&mask);
		FD_SET(fd0, &mask);
		FD_SET(fd1, &mask);

		tv.tv_sec  = 1;
		tv.tv_usec = 100000;

		n = select(setsize, &mask, (fd_set*)0, (fd_set*)0, &tv);
		if (n < 0) {
			printf("main: select returns error\n");
			exit(1);
		}
		if (FD_ISSET(fd0, &mask))
			callback0();
		if ((fd0 != fd1) && FD_ISSET(fd1, &mask) )
			callback1();

		schedule();
#ifdef MEASURE
		if (packet_count > 10000) {
			printf("Stats: %d packets at %d cycles per packet\n",
			       packet_count, packet_total / packet_count);
			return 0;
		}
#endif
        }
	return 0;
}
