/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/print_packet.h>
#include <arpa/inet.h> 
#include <stdio.h>

void print_ether(unsigned char ethaddr[ETHER_ADDR_SIZE])
{
        printf("%x:%x:%x:%x:%x:%x", 
               ethaddr[0],
               ethaddr[1],
               ethaddr[2],
               ethaddr[3],
               ethaddr[4],
               ethaddr[5]
               );
}

void print_IP(unsigned char ipaddr[4])
{
        printf("%d.%d.%d.%d", 
               ipaddr[0],
               ipaddr[1],
               ipaddr[2],
               ipaddr[3]
               );
}

void print_IP2(struct in_addr *a)
{
        print_IP((unsigned char*)a);
}

void print_ether_header(struct ether_header *e)
{
        printf("ETH{dst="); print_ether(e->ether_dhost);
        printf(", src=");   print_ether(e->ether_shost);
        printf(", type=%x}", ntohs(e->ether_type));
}

void print_ARP_header(struct arp_header *a)
{
        printf("ARP{hrd=%x, pro=%x, hln=%x, pln=%x, op=%x}",
               ntohs(a->ar_hrd),
               ntohs(a->ar_pro),
               a->ar_hln,
               a->ar_pln,
               ntohs(a->ar_op)
               );
}

void print_ether_ARP(struct ether_arp *ea)
{
        printf("EtherARP{hdr="); print_ARP_header(&(ea->ea_hdr));
        printf(", sha="); print_ether(ea->arp_sha);
        printf(", spa="); print_IP(ea->arp_spa);
        printf(", tha="); print_ether(ea->arp_tha);
        printf(", tpa="); print_IP(ea->arp_tpa);
        printf("}");
}

void print_IP_header(struct ip *ip)
{
        printf("IP{v=%d, hl=1<<%d, len=%d, id=%d, off=%x, ttl=%d, p=%d",
               ip->ip_v, 
               ip->ip_hl,
               ntohs(ip->ip_len),
               ntohs(ip->ip_id),
               ntohs(ip->ip_off),
               ip->ip_ttl,
               ip->ip_p
               );
        printf(", src="); print_IP2(&ip->ip_src);
        printf(", dst="); print_IP2(&ip->ip_dst);
        printf(", sum=%d}", ip->ip_sum);
}

void print_ICMP_echo(struct icmphdr* i)
{
        printf("ICMP_echo{type=%d, code=%d, checksum=%d, id=%d, sequence=%d}",
               i->type, 
               i->code, 
               i->checksum, 
               i->un.echo.id, 
               i->un.echo.sequence
               );
}
