/*
 * Copyright (c) 2000, 2001 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of Clack.
 * 
 * Clack is free software, also known as "open source."  You can redistribute
 * it and/or modify it under the terms of the GNU General Public License
 * (GPL), version 2, as published by the Free Software Foundation (FSF).
 * 
 * Clack is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with Clack; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 * 
 * Clack is modelled on and includes some code fragments from MIT's Click
 * system (http://www.pdos.lcs.mit.edu/click/) which is copyright (c)
 * 1999-2000 Massachusetts Institute of Technology and also GPL'ed.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

#include <clack/c/packet.h>
#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <net/bpf.h>
#include <clack/c/ethernet.h>

/*
 * Imported
 */
extern char		*interface;

/*
 * Exported
 */
unsigned char		ethaddr[ETHER_ADDR_SIZE] = { 0 };
clack_error_t		push(clack_packet_t *p);

/*
 * BPF file descriptor.
 */
int			bpf_fd;

#ifdef MEASURE
/*
 * More Imports!
 */
#include <clack/c/stats.h>
#endif

/*
 * On the output side, use the BPF directly since FreeBSD pcap is read-only.
 */
clack_error_t
init(void)
{
	int		 i, fd;
	struct	ifreq    ifr;

	for(i = 0; i < 16; i++){
		char tmp[64];
		sprintf(tmp, "/dev/bpf%d", i);
		fd = open(tmp, 1);
		if (fd >= 0){
			bpf_fd = fd;
			break;
		}
	}
	if (bpf_fd < 0) {
		printf("ToBPF: Init failed. No BPF device\n");
		return EINVAL;
	}
	
	strncpy(ifr.ifr_name, interface, sizeof(ifr.ifr_name));

	if (ioctl(bpf_fd, BIOCSETIF, (caddr_t)&ifr) < 0) {
		printf("ToBPF: Init failed. Could not ioctl(BIOCSETIF)\n");
		return errno;
	}

	if (ioctl(bpf_fd, SIOCGIFADDR, (caddr_t)&ifr) < 0) {
		printf("ToBPF: Init failed. Could not ioctl(SIOCGIFADDR)\n");
		return errno;
	}
	memcpy(ethaddr, ifr.ifr_addr.sa_data, sizeof(ethaddr));

	return 0;
}

/*
 * Push the packet out by writing it to the BPF device.
 */
clack_error_t
push(clack_packet_t *p)
{
        int		rc;
        clack_size_t	len = clack_packet_length(p);
#ifdef  MEASURE
	unsigned long long now = get_tsc();
#endif

	if ((rc = write(bpf_fd, clack_packet_data(p), len)) <= 0)
		printf("ToBPF Push: Write to fd %d filed with errno %d\n",
		       bpf_fd, errno);

	/*
	 * Just making sure we got the ref counts correct. In our simple
	 * test cases, the reference count should always be one by the
	 * time the packet is sent. Remove this at some point.
	 */
	assert(p->count == 1);
#ifdef  MEASURE
	/*
	 * Record time through the router.
	 */
	if (p->measure) {
		if (packet_count == 100 && packet_flag == 0) {
			packet_flag  = 1;
			packet_count = 0;
			packet_total = 0;
		}
		packet_count++;
		packet_total += (now - p->created);
	}
#endif
	clack_packet_release(p);

	return rc;
}

clack_error_t
fini(void)
{
	close(bpf_fd);
        return 0;
}
